#include <iostream>
#include <cstdio>
#include <vector>
#include <iterator>
#include <algorithm>
#include <utility>

char arena [102 * 102];

inline char& arena_at(int x, int y)
{
    return arena[y*102 + x];
}

const char WALL = 'X';
const char FREE = '.';
const char EXIT = 'E';
const char WIN = 'W';
const char UP = -102;
const char DOWN = 102;
const char LEFT = -1;
const char RIGHT = 1;
const char INSTR_STEP = 'S';
const char INSTR_RIGHT = 'R';
const char INSTR_LEFT = 'L';

struct Agent;
static std::vector<Agent> agents;

struct Agent {
    bool alive;
    char* arena_ptr;
    char heading;
    
    Agent(char* ptr, char heading): alive(true), arena_ptr(ptr), heading(heading) {}
    
    void sim_left() // do: right
    {
        switch (heading) {
        case UP: heading = RIGHT; break;
        case RIGHT: heading = DOWN; break;
        case DOWN: heading = LEFT; break;
        case LEFT: heading = UP; break;
        }
    }
    
    void sim_right() // do: left
    {
        switch (heading) {
        case UP: heading = LEFT; break;
        case RIGHT: heading = UP; break;
        case DOWN: heading = RIGHT; break;
        case LEFT: heading = DOWN; break;
        }
    }
    
    void sim_step() // do: go backwards
    {
        if (!alive) return;
    
        char backwards = 0;
        
        switch (heading) {
        case UP: backwards = DOWN; break;
        case RIGHT: backwards = LEFT; break;
        case DOWN: backwards = UP; break;
        case LEFT: backwards = RIGHT; break;
        }
        
        char* front = arena_ptr + heading;
        char* back = arena_ptr + backwards;
        bool front_free = *front == FREE;
        bool back_free = *back == FREE;
        
        // wall behind but no wall in front => kill this agent
        if (front_free && !back_free) {
            alive = false;
        }
        
        // wall in front but no wall behind => spawn new agent
        if (!front_free && back_free) {
            agents.push_back(Agent(back, heading));
        }
    }
};

//std::vector<Agent> agents;

void first_time_init()
{
    std::ios_base::sync_with_stdio(false);

    // clear
    for (int i = 0; i < 102*102; ++i) {
        arena[i] = '?';
    }
    
    // add inner walls
    for (int x = 0; x < 102; ++x) {
        arena_at(x, 0) = WALL;
        arena_at(0, x) = WALL;
    }
}

void draw(int w, int h)
{
    for (int y = 0; y <= h + 1; ++y) {
        for (int x = 0; x <= w + 1; ++x) {
            std::cout << arena_at(x, y);
        }
        std::cout << "|\n";
    }
    std::cout << "\n";
}

int main()
{
    first_time_init();
    
    int H;
    int W;
    while (std::cin >> H >> W) {
        // read arena
        char* exit = NULL;
        for (int y = 1; y <= H; ++y) {
            for (int x = 1; x <= W; ++x) {
                char& field = arena_at(x, y);
                std::cin >> field;
                /*if (field == EXIT) {
                    field = FREE;
                    exit = &field;
                }*/
            }
        }
        
        // add outer walls
        for (int y = 1; y <= H + 1; ++y) {
            arena_at(W + 1, y) = WALL;
        }
        for (int x = 1; x <= W + 1; ++x) {
            arena_at(x, H + 1) = WALL;
        }
        
        // add agents at exit
        agents.clear();
        agents.push_back(Agent(exit, UP));
        agents.push_back(Agent(exit, DOWN));
        agents.push_back(Agent(exit, LEFT));
        agents.push_back(Agent(exit, RIGHT));
        
        // read steps
        int n_steps;
        std::cin >> n_steps;
        std::vector<char> steps;
        for (int n = 0; n < n_steps; ++n) {
            char c;
            std::cin >> c;
            steps.push_back(c);
        }
        
        // debug draw
        draw(W, H);
        
        // simulate backwards
        for (std::vector<char>::reverse_iterator i = steps.rbegin();
            i != steps.rend(); ++i) {
            std::cout << *i << std::endl;
            
            switch(*i) {
            case INSTR_STEP:
                for (std::vector<Agent>::iterator a = agents.begin();
                    a != agents.end(); ++a) {
                    a->sim_step();
                }
                break;
            case INSTR_RIGHT:
                for (std::vector<Agent>::iterator a = agents.begin();
                    a != agents.end(); ++a) {
                    a->sim_right();
                }
                break;
            case INSTR_LEFT:
                for (std::vector<Agent>::iterator a = agents.begin();
                    a != agents.end(); ++a) {
                    a->sim_left();
                }
                break;
            }
        }
        
        /*// set places with agents as win
        for (std::vector<Agent>::iterator a = agents.begin();
            a != agents.end(); ++a) {
            if (a->alive && a->heading == UP) {
                *(a->arena_ptr) = WIN;
            }
        }
        
        // count win
        int n_win = 0;
        bool fail = false;
        for (int y = 1; y <= H; ++y) {
            for (int x = 1; x <= W; ++x) {
                if (arena_at(x, y) == WIN) n_win++;
                if (arena_at(x, y) == FREE) fail = true;
            }
        }
        
        // debug draw
        draw(W, H);
        
        // output
        if (fail) {
            std::cout << n_win << "\n";
        } else {
            std::cout << "OK\n";
        }*/
        
    }
    
    return 0;
}
